Introduction
------------

Seqtk is a fast and lightweight tool for processing sequences in the FASTA or
FASTQ format. It seamlessly parses both FASTA and FASTQ files which can also be
optionally compressed by gzip.

Written by Heng Li.

This version was reduced to perform only the `comp` function and integrated to mapDamage.
The original version can be found here  
https://github.com/lh3/seqtk

